<?php
/*
 *  Copyright (C) 2006 CSIRO Australia
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  - Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *  
 *  - Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  
 *  - Neither the name of the CSIRO nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE ORGANISATION OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * \file AnnodexWriter.php
 * \author Chris Ford
 * \date January 2006
 */

require_once("phpannodex/Annodex.php");

/**
 * \class AnnodexWriter
 * A class for writing Annodex media.
 */
class AnnodexWriter extends Annodex {

  /** The buffer size used when writing (in bytes). */
  private $buffer_size;

  /** The default buffer size used (in bytes). */
  const DEFAULT_BUFFER_SIZE = 102400;

  /**
   * The constructor.
   *
   * This method initialises all Annodex importers, and then calls Annodex's
   * constructor to create a new AnnodexWriter. 
   */
  public function __construct() {
  
    parent::__construct( Annodex::WRITE ); 
    anx_init_importers("*/*");
    $this->buffer_size = self::DEFAULT_BUFFER_SIZE;

    return;
  }

  /**
   * A standard setter method.
   * \param $size the size in bytes that the buffer will be set to.
   */
  public function set_buffer_size( $size ) {
    $this->buffer_size = $size;
    return;
  }

  /**
   * A standard getter method.
   * \return the current size of the buffer.
   */
  public function get_buffer_size() {
    return $this->buffer_size;
  }
  
  /** 
   * Inserts CMML into the AnnodexWriter.
   * 
   * \param $at_time the time in seconds to schedule the clip.
   * \param $cmml a valid CMML element.
   */
  public function insert_cmml( $at_time, $cmml ) {
    return anx_insert_cmml( $this->get_anx_handle(), $at_time, 
        $cmml, strlen( $cmml ) );
  }
    
  /** 
   * Inserts CMML into the AnnodexWriter, as represented by a SimpleXMLElement.
   * 
   * \param $at_time the time in seconds to schedule the clip.
   * \param $simple_xml a SimpleXMLElement representing a valid CMML element.
   */
  public function insert_simplexml( $at_time, SimpleXMLElement $simple_xml ) {
    return $this->insert_cmml( $at_time, $simple_xml->asXML() );
  }

  /** 
   * Inserts CMML into the AnnodexWriter, as represented by a DOMNode.
   * 
   * \param $at_time the time in seconds to schedule the clip.
   * \param $node a DOMNode representing a valid CMML element.
   */
  public function insert_dom_node( $at_time, DOMNode $node ) {
    return $this->insert_simplexml( $at_time, simplexml_import_dom( $node ));
  }

  /**
   * Imports a file into the AnnodexWriter.
   *
   * All parameters other than the name of the file are optional.
   *
   * \param $filename the file to import.
   * \param $id the id of the file.
   * \param $content_type the content-type of the whole import file. If not 
   * specified will be deducded from the file extension.
   * \param $start a time in seconds to begin importing data from. Defaults to
   * the start of the file.
   * \param $end a time in seconds to finish importing data from. Defaults to
   * the very end of the file.
   * \param $flags writer import flags.
   */
  public function import( $filename, $id=NULL, $content_type=NULL, 
        $start=self::START, $end=self::END, $flags=0 ) {

    /* Returns true for success */
    $result = anx_write_import( $this->get_anx_handle(), $filename, 
        $id, $content_type, $start, $end, $flags );
    $this->check_anx_error();
    return;
  }

  /**
   * Outputs a block of binary data. The data will not always be the size of the
   * current buffer, because there may not be a buffer's worth of data still
   * left to write.
   *
   * \return a binary string of Annodex media on success, FALSE on failure.
   */
  public function write_output() {

    /* Returns false on failure */
    return anx_write_output( $this->get_anx_handle(), 
      $this->buffer_size );

  }


}

